﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Net;
using System.IO;
using System.IO.IsolatedStorage;
using System.Text;
using System.Windows.Browser;
using SilverlightReporter.ReportService; // reference to ReportService2005 endpoint
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace SilverlightReporter
{
    public partial class Page : UserControl
    {
        #region Class Members
        private HtmlDocument            _doc;
        private List<EntityParameter>   _userParameters;
        #endregion Class Members
        #region Properties
        /// <summary>
        /// Constructs the report URL address
        /// </summary>
        private string ReportUrl
        {
            get
            {
                StringBuilder urlBuilder = new StringBuilder();
                urlBuilder.Append("http://localhost/reportserver");
                urlBuilder.Append("?");
                urlBuilder.Append(this.ReportPath);

                // get a reference to the grid data source
                _userParameters = (List<EntityParameter>)grdParams.ItemsSource;

                // populate the parameters collection
                foreach (EntityParameter ep in _userParameters)
                {
                    string[] parameters = ep.Value.Split(new char[] { ',' });
                    foreach (string p in parameters)
                    {
                        urlBuilder.Append(String.Format("&{0}={1}", ep.Name, p));
                    }
                }
                urlBuilder.Append(@"&rs:Command=Render");
                if (radHtmlFragment.IsChecked==true)
                {
                    urlBuilder.Append(@"&rc:Toolbar=False");
                    urlBuilder.Append(@"&rc:HtmlFragment=True");
                    urlBuilder.Append(@"&rc:Zoom=75");
                }
                string url = urlBuilder.ToString();
                return url;
            }
        }
        /// <summary>
        /// Returns the report path
        /// </summary>
        private string ReportPath
        {
            get
            {
                return String.Format("{0}/{1}", txtFolder.Text, lstReports.SelectedItem.ToString());
            }
        }
        /// <summary>
        /// Returns the URL to the ReportService2005 execution endpoint
        /// </summary>
        private string ReportServiceAddress
        {
            get
            {
                return String.Format("{0}/ReportService2005.asmx", txtServerUrl.Text);
            }
        }
        #endregion 
        #region Methods
        /// <summary>
        /// Loads reports from the report server
        /// </summary>
        private void LoadReports()
        {
            Binding binding = new BasicHttpBinding(); // Silverlight supports basic binding only
            EndpointAddress address = new EndpointAddress(this.ReportServiceAddress);
            ReportingService2005SoapClient rs = new ReportingService2005SoapClient(binding, address);
            // Since Silverlight supports asynchronous WS calls only, we need to set an event handler
            // for every method invocation
            rs.ListChildrenCompleted += new EventHandler<ListChildrenCompletedEventArgs>(ListChildrenCompleted);
            // Call ListChildren API
            rs.ListChildrenAsync(txtFolder.Text, false);
        }
        /// <summary>
        /// The asynch event handler for ListChildren
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void ListChildrenCompleted(object sender, ListChildrenCompletedEventArgs e)
        {
            CatalogItem[] items = e.CatalogItems; // get the result
            foreach (CatalogItem item in items)
            {
                if (item.Type == ReportService.ItemTypeEnum.Report && item.Name.Contains("Chart")) lstReports.Items.Add(item.Name);
            }
            // Add reports to list
            if (lstReports.Items.Count > 0)
            {
                lstReports.SelectedIndex = 0;
                btnRun.IsEnabled = true;
            }
        }
        /// <summary>
        /// Fired when the user selects a report
        /// </summary>
        private void LoadReportParameters()
        {
            Binding binding = new BasicHttpBinding();
            EndpointAddress address = new EndpointAddress(this.ReportServiceAddress);
            ReportingService2005SoapClient rs = new ReportingService2005SoapClient(binding, address);
            rs.GetReportParametersCompleted += new EventHandler<GetReportParametersCompletedEventArgs>(GetReportParametersCompleted);
            rs.GetReportParametersAsync(this.ReportPath, null, false, null, null);
        }
        /// <summary>
        /// The async handler for the GetParameters API
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void GetReportParametersCompleted(object sender, GetReportParametersCompletedEventArgs e)
        {
            // _userParameters is a collection of EntityParameters object that will hold the parameters
            // so we can bind DataGrid to the collection
            _userParameters= new List<EntityParameter>();
            foreach (ReportParameter p in e.Parameters)
            {
                _userParameters.Add(new EntityParameter(p.Name, 
                    p.DefaultValues==null?String.Empty: String.Join(",", p.DefaultValues)));
            }
            // Bind parameters to grid
            grdParams.ItemsSource = _userParameters;
            btnRun.IsEnabled = true;
        }
        /// <summary>
        /// The callback method for ReportRender
        /// </summary>
        /// <param name="ar"></param>
        private void ReportRenderResponseCompleted(IAsyncResult ar)
        {
            string reportPayload = null;
            object[] State = (object[])ar.AsyncState;
            WebRequest webReq = (WebRequest)State[0];

            //get the response
            WebResponse response = webReq.EndGetResponse(ar);
            using (StreamReader reader = new StreamReader(response.GetResponseStream()))
            {
                reportPayload = reader.ReadToEnd();
            }
            // Set the innerHTML property of the reportPlaceholder DIV
            _doc.GetElementById("reportPlaceholder").SetProperty("innerHTML", reportPayload);
        }
        #endregion Methods
        #region Events
        public Page()
        {
            InitializeComponent();
            // obtain a reference to the browser DOM object
            _doc = HtmlPage.Document;
            btnRun.IsEnabled = false;
            // Load the user preferences (server Url and path) from isolated storage
            LoadUserSettings();
            // Load reports from the report server
            LoadReports();
        }
        /// <summary>
        /// The Run Reports click event handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            string url = this.ReportUrl;

            if (radHtmlFragment.IsChecked == true)
            {
                // If HTML Fragment option selected, use System.NET.WebRequest to send an URL request
                WebRequest request = WebRequest.Create(new Uri(url));
                IAsyncResult arResp = request.BeginGetResponse(new AsyncCallback(this.ReportRenderResponseCompleted), new object[] 
                    { request, lstReports.SelectedItem.ToString() });
            }
            else
            {
                // If the IFrame option is selected set the IFRAME SRC property to the report URL
                _doc.GetElementById("iReport").SetProperty("src", this.ReportUrl);
            }
            // Since there isn't a control Unload event, save the user preferences here
            SaveUserSettings();
        }
 
        /// <summary>
        /// The SelectionChanged event handler of lstReports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstReports_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            btnRun.IsEnabled = false;
            LoadReportParameters();
        }
        #endregion Events
        #region IsolatedStorage
        /// <summary>
        /// Read user preferences from isolated storage
        /// </summary>
        private void LoadUserSettings()
        {
            string data = null;
            IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication();
            try
            {
                using (IsolatedStorageFileStream isoStream = new IsolatedStorageFileStream("app.config", 
                        FileMode.Open, iso))
                {
                    using (StreamReader reader = new StreamReader(isoStream))
                    {
                        data = reader.ReadToEnd();
                    }
                }
            }
            catch { } // no big deal if we fail

            if (data != null)
            {
                string[] settings = data.Split(new char[] {'~'});
                txtServerUrl.Text   = settings[0];
                txtFolder.Text      = settings[1];
            }
        }
        /// <summary>
        /// Saves the user preferences in isolated storage
        /// </summary>
        private void SaveUserSettings()
        {
            string data = String.Format("{0}~{1}", txtServerUrl.Text, txtFolder.Text);
            IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForApplication();
            using (IsolatedStorageFileStream isoStream = iso.CreateFile("app.config"))
            {
                using (StreamWriter writer = new StreamWriter(isoStream))
                {
                    writer.Write(data);
                }
            }

        }
        #endregion IsolatedStorage
    }
}
