using System;
using System.IO;
using Drawing = System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Specialized;
using Microsoft.ReportingServices.OnDemandReportRendering;

namespace Prologika.RS.Extensibility.ProgressTrackerCRI
{
    /// <summary>
    /// The main class for the custom report item run-time component.
    /// The report processor first calls the GenerateReportItemDefinition method 
    /// and then calls the EvaluateReportItemInstance method to get the rendered 
    /// report item.
    /// </summary>
    public class ProgressTrackerRenderer : ICustomReportItem
    {
        #region Class members
        private ProgressTracker _progress = null;
        private Microsoft.ReportingServices.ReportRendering.ImageMapAreasCollection _imageMap;
        #endregion

        public void GenerateReportItemDefinition(CustomReportItem cri)
        {
            // Create the Image Definition object that will be 
            // used to render the custom report item
            cri.CreateCriImageDefinition();
            Image progressImage = (Image)cri.GeneratedReportItem;
        }

        public void EvaluateReportItemInstance(CustomReportItem cri)
        {
            // Get the Image definition
            Image progressImage = (Image)cri.GeneratedReportItem;

            // Render the image for the custom report item
            progressImage.ImageInstance.ImageData = DrawImage(cri);
        }


        public Action Action
        {
            get
            {
                return null;
            }
        }

        /// <summary>
        /// Renders the CRI image
        /// </summary>
        /// <returns></returns>
        private byte[] DrawImage(CustomReportItem cri)
        {
            byte[] imageData;
            // initialize CRI from properies in RDL
            Initialize(cri);
            CreateAction(cri); // creates a hyperlink action

            Drawing.Bitmap image = new Drawing.Bitmap(_progress.Width, 
                _progress.Height, PixelFormat.Format32bppArgb);
            //bmp.SetResolution(300, 300);
            System.Drawing.Graphics graphics = Drawing.Graphics.FromImage(image);
            Drawing.Color backgroundColor = cri.Style.BackgroundColor.Value.ToColor();
            if (backgroundColor.IsEmpty) backgroundColor = Drawing.Color.White;
            graphics.Clear(backgroundColor);

            _progress.DrawControl(graphics);
            // create a memory stream to save the image
            using (MemoryStream stream = new MemoryStream())
            {
                image.Save(stream, ImageFormat.Bmp);
                imageData = new byte[(int)stream.Length];
                stream.Seek(0, SeekOrigin.Begin);
                stream.Read(imageData, 0, (int)stream.Length);
            }
            return imageData;
            
        }
        /// <summary>
        /// Creates a hyperlink action5 if the ProgressBar Hyperlink property is set
        /// </summary>
        private void CreateAction (CustomReportItem cri)
        {
            if (!string.IsNullOrEmpty(_progress.Hyperlink))
            {
                Image progressImage = (Image)cri.GeneratedReportItem;
                ActionInfoWithDynamicImageMap imageMap = progressImage.ImageInstance.CreateActionInfoWithDynamicImageMap();
                Action action = imageMap.CreateHyperlinkAction();
                action.Instance.HyperlinkText = _progress.Hyperlink;
                // top left and bottom right coordinates (last two are in percentage points!)
                float[] coordinates = new float[] { 20, 5, 90, 90 };
                imageMap.CreateImageMapAreaInstance(ImageMapArea.ImageMapAreaShape.Rectangle, coordinates);
            }
        }


        /// <summary>
        /// Initialize the bar from the custom properties in RDL
        /// </summary>
        private void Initialize(CustomReportItem cri)
        {
            if (null == _progress)
            {
                _progress = new ProgressTracker();
            }
            _progress.Alpha = Int32.Parse(LookupCustomProperty(cri.CustomProperties, Shared.PROP_ALPHA, 255).ToString());
            _progress.ProgressColor = cri.Style.Color.Value.ToColor();
            _progress.BorderColor = cri.Style.Border.Color.Value.ToColor();
            _progress.Font = Shared.GetDrawingFontFromReportFont(cri.Style.FontFamily.Value, cri.Style.FontStyle.Value.ToString(),
                cri.Style.FontWeight.Value.ToString(), (float)(cri.Style.FontSize.Value.ToPoints()));
            _progress.Maximum = float.Parse(LookupCustomProperty(cri.CustomProperties, Shared.PROP_MAXIMUM, 1f).ToString());
            _progress.Minimum = float.Parse(LookupCustomProperty(cri.CustomProperties, Shared.PROP_MINIMUM, -1f).ToString());
            _progress.Value = float.Parse(LookupCustomProperty(cri.CustomProperties, Shared.PROP_VALUE, 0).ToString());
            _progress.ShowValue = bool.Parse(LookupCustomProperty(cri.CustomProperties, Shared.PROP_SHOW_VALUE, "true").ToString());
            _progress.Hyperlink = (string)LookupCustomProperty(cri.CustomProperties, Shared.PROP_HYPERLINK, String.Empty);

            // size the control
            _progress.Height = (int)(cri.Height.ToInches() * Shared.DPI);
            _progress.Width = (int)(cri.Width.ToInches() * Shared.DPI);
        }

        /// <summary>
        /// Helper method to look up a property from the report definition
        /// </summary>
        /// <param name="customProperties">Custom property collection</param>
        /// <param name="name">Property name to look up</param>
        /// <param name="defaultvalue">The default property value if the property default value hasn't changed</param>
        /// <returns>Property value</returns>
        private object LookupCustomProperty(CustomPropertyCollection customProperties, string name, object defaultValue)
        {
            object customPropertyValue = defaultValue;

            if (customProperties == null || customProperties.Count == 0)
                return defaultValue;

            CustomProperty customProperty = customProperties[name];

            if (customProperty != null)
            {
                // If the property exists check to see if it is an expression
                // if it is an expression return the evaluated instance
                // otherwise the definition value can be returned
                if (customProperty.Value.IsExpression)
                    customPropertyValue = customProperty.Instance.Value;
                else
                    customPropertyValue = customProperty.Value.Value;
            }

            return customPropertyValue;
        }
    }
}
