﻿#region Disclaimer by Teo Lachev "Applied Microsoft SQL Server 2008 Reporting Services"
/*============================================================================
  File:     ReportDefinitionCustomizationExtension.cs

  Summary:  Demonstrates using RDLOM to change report definitions at run time. 
--------------------------------------------------------------------
  This code sample uses the Microsoft.ReportingServices.RdlObjectModel assembly
  which represents the Microsoft RDL Object Model. The Microsoft RDL Object Model
  is an internal and unsupported component of Reporting Services. As such, Microsoft may 
  change or discontinue RDLOM. Use RDLOM at your own risk.
===========================================================================*/
#endregion
using System;
using System.Collections.Generic;
using System.Xml;
using System.Text;
using System.IO;
using Microsoft.ReportingServices.Interfaces;
using Microsoft.ReportingServices.RdlObjectModel;
using Microsoft.ReportingServices.RdlObjectModel.Serialization;

namespace Prologika.RS.Extensibility
{
    public class ReportDefinitionCustomizationExtension : IReportDefinitionCustomizationExtension
    {
        internal const string PRIVILEGED_USER = "PrivilegedUser";
        private string _user = null;
        Microsoft.ReportingServices.RdlObjectModel.Report _report = null;
        RdlSerializer _serializer = new RdlSerializer();

        #region IReportDefinitionCustomizationExtension Members

        public bool ProcessReportDefinition(
            byte[] reportDefinition,
            IReportContext reportContext,
            IUserContext userContext,
            out byte[] reportDefinitionProcessed,
            out IEnumerable<RdceCustomizableElementId> customizedElementIds)
        {
            MemoryStream mstream = null;
            if (String.Compare(_user, userContext.UserName, StringComparison.OrdinalIgnoreCase) != 0)
            {
                // not the right user so return the original RDL without any changes
                reportDefinitionProcessed = null;
                customizedElementIds = null;
                return false;
            }
            // use RDLOM to deserialize the original def as a report object
            using (mstream = new MemoryStream(reportDefinition))
            {
                mstream.Position = 0;
                _report = _serializer.Deserialize(mstream);
            }

            AddQueryFields();   // add query fields
            AddDatasetFields(); // add dataset fields
            AddColumns();       // add tablix columns

            // serialize the report payload to byte array
            using (mstream = new MemoryStream())
            {
                _serializer.Serialize(mstream, _report);
                reportDefinitionProcessed = mstream.ToArray();
            }

            // let RS know what's been changed
            List<RdceCustomizableElementId> ids = new List<RdceCustomizableElementId>();
            ids.Add(RdceCustomizableElementId.Body);
            ids.Add(RdceCustomizableElementId.DataSets);
            customizedElementIds = ids;

            return true; // RDL is customized
        }

        /// <summary>
        /// Add query fields
        /// </summary>
        /// <param name="?"></param>
        private void AddQueryFields()
        {
            // add BaseRate field to the dataset query
            _report.DataSets[0].Query.CommandText = _report.DataSets[0].Query.CommandText.Value.Replace("LoginID", "LoginID, BaseRate ");
        }
        /// <summary>
        /// Add dataset fields
        /// </summary>
        /// <param name="?"></param>
        private void AddDatasetFields()
        {
            // add BaseRate field to the dataset
            Field baseRate = new Field();
            baseRate.Name = "BaseRate";
            baseRate.DataField = "BaseRate";
            _report.DataSets[0].Fields.Add(baseRate);
        }
        /// <summary>
        /// Add tablix columns
        /// </summary>
        private void AddColumns()
        {
            TablixMember member;
            TablixColumn column;
            TablixRow row;
            TablixCell cell;

            // add a new column
            Tablix tablix = (Tablix)_report.Body.ReportItems[0];
            member = new TablixMember();
            tablix.TablixColumnHierarchy.TablixMembers.Add(member);
            column = new TablixColumn();
            column.Width = new ReportSize(1);
            tablix.TablixBody.TablixColumns.Add(column);

            // add a new header cell
            row = tablix.TablixBody.TablixRows[0];
            cell = new TablixCell();
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "txtBaseRateCaption";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("Base Rate");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingTop =
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingBottom =
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingBottom =
            ((Textbox)cell.CellContents.ReportItem).Style.PaddingBottom = new ReportExpression<ReportSize>("2pt");

            ((Textbox)cell.CellContents.ReportItem).Style.Border = new Border();
            ((Textbox)cell.CellContents.ReportItem).Style.Border.Color = new ReportExpression<ReportColor>("LightGrey");
            ((Textbox)cell.CellContents.ReportItem).Style.Border.Style = new ReportExpression<BorderStyles>("Solid");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.FontWeight = new ReportExpression<FontWeights>("Bold");

            // add a new detail cell
            cell = new TablixCell();
            row = tablix.TablixBody.TablixRows[1];
            row.TablixCells.Add(cell);
            cell.CellContents = new CellContents();
            cell.CellContents.ReportItem = new Textbox();
            ((Textbox)cell.CellContents.ReportItem).Name = "txtBaseRate";
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns.Add(new TextRun());
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Value = new ReportExpression("=Fields!BaseRate.Value");
            ((Textbox)cell.CellContents.ReportItem).Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Style.Border = new Border();
            ((Textbox)cell.CellContents.ReportItem).Style.Border.Color = new ReportExpression<ReportColor>("LightGrey");
            ((Textbox)cell.CellContents.ReportItem).Style.Border.Style = new ReportExpression<BorderStyles>("Solid");
            ((Textbox)cell.CellContents.ReportItem).Style.BackgroundColor = new ReportExpression<ReportColor>("=Iif(RowNumber(Nothing) Mod 2, \"AliceBlue\", \"White\")");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].Style.TextAlign = new ReportExpression<TextAlignments>("Right");
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style = new Style();
            ((Textbox)cell.CellContents.ReportItem).Paragraphs[0].TextRuns[0].Style.Format = new ReportExpression("C");

            // serialize to disk (for testing purposes only
            //using (FileStream os = new FileStream(@"C:\temp.rdl", FileMode.Create))
            //{
            //    _serializer.Serialize(os, _report);
            //}
        }
        #endregion

        #region IExtension Members

        public string LocalizedName
        {
            get { return "Prologika Report Definition Customization Extension Sample"; }
        }

        public void SetConfiguration(string configuration)
        {
            // Retrieve the identity of the privileged user who is authorized to see sensitive data
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(configuration);
            if (doc.DocumentElement.Name == "RDCEConfiguration")
            {
                foreach (XmlNode child in doc.DocumentElement.ChildNodes)
                {
                    switch (child.Name)
                    {
                        case PRIVILEGED_USER: _user = child.InnerText; break;
                        default: throw new Exception("Unrecognized configuration element.");
                    }
                }
            }
            else
                throw new Exception("Loading config data.");
        }

        #endregion
    }
}
